@@ -9,29 +9,30 @@
 #include "waylandintegration.h"
 
 #include <QDebug>
-#include <QWidget>
-#include <QGuiApplication>
 #include <QExposeEvent>
+#include <QGuiApplication>
+#include <QWidget>
 
-#include <KWayland/Client/connection_thread.h>
-#include <KWayland/Client/plasmawindowmanagement.h>
-#include <KWayland/Client/registry.h>
-#include <KWayland/Client/plasmashell.h>
-#include <KWayland/Client/compositor.h>
-#include <KWayland/Client/surface.h>
 #include <KWayland/Client/blur.h>
+#include <KWayland/Client/compositor.h>
+#include <KWayland/Client/connection_thread.h>
 #include <KWayland/Client/contrast.h>
+#include <KWayland/Client/plasmashell.h>
+#include <KWayland/Client/plasmawindowmanagement.h>
 #include <KWayland/Client/region.h>
+#include <KWayland/Client/registry.h>
 #include <KWayland/Client/slide.h>
+#include <KWayland/Client/surface.h>
 
 WindowEffects::WindowEffects()
-    : QObject(),
-      KWindowEffectsPrivate()
+    : QObject()
+    ,  KWindowEffectsPrivateV2()
 {
 }
 
 WindowEffects::~WindowEffects()
-{}
+{
+}
 
 QWindow *WindowEffects::windowForId(WId wid)
 {
@@ -71,7 +72,7 @@ void WindowEffects::releaseWindow(QWindow *window)
 bool WindowEffects::eventFilter(QObject *watched, QEvent *event)
 {
     if (event->type() == QEvent::Expose) {
-        auto ee = static_cast<QExposeEvent*>(event);
+        auto ee = static_cast<QExposeEvent *>(event);
 
         if ((ee->region().isNull())) {
             return false;
@@ -262,6 +263,29 @@ void WindowEffects::enableBackgroundContrast(QWindow *window, bool enable, qreal
     }
 }
 
+void WindowEffects::setBackgroundFrost(QWindow *window, QColor color, const QRegion &region)
+{
+    if (!WaylandIntegration::self()->waylandContrastManager()) {
+        return;
+    }
+
+    KWayland::Client::Surface *surface = KWayland::Client::Surface::fromWindow(window);
+    if (!surface) {
+        return;
+    }
+    if (!color.isValid()) {
+        WaylandIntegration::self()->waylandContrastManager()->removeContrast(surface);
+        return;
+    }
+
+    auto backgroundContrast = WaylandIntegration::self()->waylandContrastManager()->createContrast(surface, surface);
+    backgroundContrast->setRegion(WaylandIntegration::self()->waylandCompositor()->createRegion(region, nullptr));
+    backgroundContrast->setFrost(color);
+    backgroundContrast->commit();
+
+    WaylandIntegration::self()->waylandConnection()->flush();
+}
+
 #if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 67)
 void WindowEffects::markAsDashboard(WId window)
 {
